"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const state_tracker_1 = require("./state-tracker");
const use_injection_1 = require("./use-injection");
/**
 * Creates an object that contains a provider component that can be used to pass
 * the container down to child components, and an injector decorator that can be used
 * to inject services into a component.
 */
function createInjection(defaultContainer) {
    // Create a react context to allow sharing of the given container.
    const context = react_1.createContext(defaultContainer);
    const { Provider, Consumer } = context;
    return {
        /**
         * The context of the injection.
         */
        context,
        /**
         * Provides child components with the given container.
         */
        InjectionProvider: class InjectionProvider extends react_1.default.Component {
            constructor(props) {
                super(props);
                // Make sure StateTracker has been bound to the current container.
                state_tracker_1.StateTracker.bindToContainer(props.container);
            }
            render() {
                return react_1.default.createElement(Provider, { value: this.props.container }, this.props.children);
            }
        },
        /**
         * A wrapped version of the `useInjection` hook that uses the current context.
         */
        useInject(inject) {
            return use_injection_1.useInjection(context, inject);
        },
        /**
         * Returns a function that will create a component that will have the requested services
         * injected as props.
         */
        injectComponent(inject, stateMapper) {
            // Return an full-bodied function to prevent syntax errors due to JSX conflicts.
            return function (Comp) {
                const injector = class extends react_1.Component {
                    constructor() {
                        super(...arguments);
                        this.handleUpdate = () => {
                            // Trigger render.
                            this.setState({});
                        };
                    }
                    componentDidMount() {
                        this.bindHandlers();
                    }
                    componentDidUpdate() {
                        this.bindHandlers();
                    }
                    componentWillUnmount() {
                        this.unbindHandlers();
                    }
                    render() {
                        return (react_1.default.createElement(Consumer, null, container => {
                            // Extract container from props or consumer.
                            container = this.props.container || container;
                            // Ensure that we got a container from somewhere.
                            if (!container) {
                                throw new Error('No container was provided, either provide a default one or use a ContainerProvider component.');
                            }
                            // Unbind handlers first to prevent memory leaks.
                            this.unbindHandlers();
                            // Get the current state tracker.
                            if (container.isBound(state_tracker_1.StateTracker)) {
                                this.stateTracker = container.get(state_tracker_1.StateTracker);
                            }
                            // Get the necessary services that we need to inject.
                            const services = {};
                            for (const key of Object.keys(inject)) {
                                services[key] = container.get(inject[key]);
                            }
                            // If given a mapping function, map the service state to props directly.
                            let stateMap;
                            if (stateMapper) {
                                stateMap = stateMapper(services);
                            }
                            // Init the wrapper component with the given props and services.
                            // @ts-ignore
                            return react_1.default.createElement(Comp, Object.assign({}, this.props, services, stateMap));
                        }));
                    }
                    bindHandlers() {
                        if (this.stateTracker) {
                            this.stateTracker.handlers.add(this.handleUpdate);
                        }
                    }
                    unbindHandlers() {
                        if (this.stateTracker) {
                            this.stateTracker.handlers.delete(this.handleUpdate);
                        }
                    }
                };
                // Give the component a useful name for debugging.
                if (process.env.NODE_ENV === 'development') {
                    Object.defineProperty(injector, 'name', {
                        value: `Injector(${Comp.displayName || Comp.name || 'Component'})`,
                    });
                }
                return injector;
            };
        },
    };
}
exports.createInjection = createInjection;
//# sourceMappingURL=create-injection.js.map